// C Source File
// Created 27/07/2003; 11:53:11

#include "mkcomplib.h"

/*===============================================================================================
== Create a Ti-OS variable 
=================================================================================================
== hsym_var_src : hsym pointer to the source variable
== name_var_dest : string of the name of the variable to create : if NULL, it replaces the source
variable
== h : handle to associate to the new variable
== error : pointer to return the error code
=================================================================================================
== return the HSym structure of the new var
===============================================================================================*/
HSym mk_var(HSym hsym_var_src, const char * name_var_dest, HANDLE h, CompErr * error) {

  BOOL arch=FALSE;
  SYM_ENTRY * SymPtr;
  HSym hsym;
  SYM_ENTRY * sym_var_src=DerefSym(hsym_var_src);

  if (name_var_dest==NULL) {
    if (sym_var_src->flags.bits.archived) {
      EM_abandon(sym_var_src->handle);
      sym_var_src->flags.bits.archived=0;
      arch=TRUE;
    } else {
      HeapFree(sym_var_src->handle);
    }
    sym_var_src->handle=h;
    if (arch)
      if (!(EM_moveSymToExtMem(NULL,hsym_var_src)))
        *error=COMP_ERR_CANT_REARCH;  //can't rearchive the var
    return hsym_var_src;
  }
  
  hsym=SymFind(SYMSTR(name_var_dest));

  if (hsym.folder!=H_NULL) { //var found
    SymPtr=DerefSym(hsym);
    if (SymPtr->flags.bits.archived) { //variable archived
      arch=TRUE;  //need to rearchive after created
      EM_abandon (SymPtr->handle);
    } else {
      HeapFree(SymPtr->handle);
    }
    //delete the var
    SymPtr->handle=0;
    SymDel(SYMSTR(name_var_dest));
  }

  hsym=SymAdd(SYMSTR(name_var_dest));
  
  if (hsym.folder!=H_NULL) {
    DerefSym(hsym)->handle=h;
    
    if (arch) //the variable was archived
      if (!EM_moveSymToExtMem(NULL,hsym))
        *error=COMP_ERR_CANT_REARCH;  //can't rearchive the var
  } else {
    *error=COMP_ERR_CANT_ADD;  //can't add the var
  }

  return hsym;
}

