// C Source File
// Created 27/07/2003; 11:59:01

#include "mkcomplib.h"

/*===============================================================================================
== Ziplib Decompression of a variable in an other (or the same) variable
=================================================================================================
== hsym_var_src : HSym structure of the var to compress
== name_var_dest : name of the variable where the compressed datas will be stored
         if it's NULL, compress in the same variable
== error : pointer to return the error code
==================================================================================================
== return the hsym structure created, else HS_NULL
===============================================================================================*/

HSym dzip_hsym_var(HSym hsym_var_src, const unsigned char * name_var_dest, CompErr * error) {
  HANDLE h_var;
  unsigned short length;
  unsigned char * data;

  TREE * tree=NULL;
  short nL;
  unsigned short file_len;
  short byte_offset;
  short bit_offset;

  HANDLE h;
  unsigned char * dest;
  
  if (hsym_var_src.folder==H_NULL) {
    *error=COMP_ERR_NO_SRC_VAR;
    return HS_NULL;
  }
    
  h_var=DerefSym(hsym_var_src)->handle;
  data=HLock(h_var);
  if (data==NULL) {
    *error=COMP_ERR_MEM;
    return HS_NULL;
  }
  length=*(unsigned short *)data;

  *error=study_dzip(data+2,length,&tree,&nL,&file_len,&byte_offset,&bit_offset);
  
  if (*error!=COMP_NO_ERR) {
    HeapUnlock(h_var);
    return HS_NULL;
  }

  h=HeapAlloc(file_len+2);
  if (h==H_NULL) {
    free(tree);
    HeapUnlock(h_var);
    *error=COMP_ERR_MEM;
    return HS_NULL;
  }
  dest=HLock(h);
  
  *error=mk_dzip(data+2,dest,NULL,0,tree,nL,file_len,byte_offset,bit_offset);

  HeapUnlock(h_var);

  if (*error!=COMP_NO_ERR) {
    HeapFree(h);
    return HS_NULL;
  }
  
  HeapUnlock(h);  

  return mk_var(hsym_var_src,name_var_dest,h,error);
}

