// C Source File
// Created 27/07/2003; 11:54:15

#include "mkcomplib.h"

/*===============================================================================================
== Ziplib compression of a variable in an other (or the same) variable
=================================================================================================
== hsym_var_src : HSym structure of the var to compress
== name_var_dest : name of the variable where the compressed datas will be stored
         if it's NULL, compress in the same variable
== error : pointer to return the error code
==================================================================================================
== return the hsym structure created, else HS_NULL
===============================================================================================*/

HSym zip_hsym_var(HSym hsym_var_src, const unsigned char * name_var_dest, CompErr *error) {
  HANDLE h_var;
  unsigned char * data;
  unsigned char type;
  unsigned short length=0;

  TREE * tree=NULL;
  short          nL;
  unsigned short comp_len;
  short          offset;

  HANDLE h;
  unsigned char * dest;
  
  if (hsym_var_src.folder==H_NULL) {
    *error=COMP_ERR_NO_SRC_VAR;
    return HS_NULL;
  }

  h_var=DerefSym(hsym_var_src)->handle;
  data=HLock(h_var);
  if (data==NULL) {
    *error=COMP_ERR_MEM;
    return HS_NULL;
  }

  length=*(unsigned short *)data;

  type=conv_type_TIOS_filelib(data[length+1]);

  *error=study_zip(data+2,length,&tree,&nL,&comp_len,&offset,FALSE);

  if (*error!=COMP_NO_ERR) {
    HeapUnlock(h_var);
    return HS_NULL;
  }  
  
  h=HeapAlloc(comp_len+2);
  if (h==H_NULL) {
    free(tree);
    HeapUnlock(h_var);
    *error=COMP_ERR_MEM;
    return HS_NULL;
  }
  dest=HLock(h);
  
  *error=mk_zip(data+2,length,dest,tree,nL,comp_len,offset,type,FALSE);

  HeapUnlock(h_var);

  if (*error!=COMP_NO_ERR) {
    HeapFree(h);
    return HS_NULL;
  }

  HeapUnlock(h);

  return mk_var(hsym_var_src,name_var_dest,h,error);
}
