
#ifndef __hiblib__
#define __hiblib__

/**************** Main engine ********************/

#define h_isPic(hfile) hfile->isPic

BOOL hl_parse(h_File * hfile, const unsigned char * text, unsigned short size_text);

BOOL hl_layout(h_File * hfile, short width, BOOL allowBigObject, h_Font * fonttab, short nb_font, FctIsComp iscomp, FctUnComp uncomp);

void hl_freehFile(h_File * hfile);

void hl_lockhFile(h_File * hfile);

void hl_unlockhFile(h_File * hfile);

void hl_setScreen(h_File * hfile, void * screen, short x, short y, short width, short height);

void hl_screenCopy(h_Screen dst, h_Screen src);

void hl_drawPage(short no_line, h_File * hfile);

#define hl_refreshScreen(hfile)   {hl_fillFrame(hfile->screen.pos.x,hfile->screen.pos.y,hfile->screen.pos.x+hfile->screen.size.width-1,hfile->screen.pos.y+hfile->screen.size.height-1,HGRAPHMODE_WHITE,hfile->screen.mem);hl_screenCopy(hfile->screen,hfile->buffer);}

#define hl_changeScreen(hfile,screen,x,y,width,height) {hl_setScreen(hfile,screen,x,y,width,height);hl_updateBuffers(hfile);hl_refreshScreen(hfile);}

#define hl_initScreen(hfile,screen,x,y,width,height) {hl_setScreen(hfile,screen,x,y,width,height);hl_drawPage(0, hfile);}

#define hl_redraw(hfile)  {hl_drawPage(-1,hfile);}

/**************** Scrolling capabilities ********************/

void hl_updateBuffers(h_File * hfile);

void hl_goPixelDown(h_File * hfile);

void hl_goPixelUp(h_File * hfile);

void hl_goPixelLeft(h_File * hfile);

void hl_goPixelRight(h_File * hfile);

#define hl_goLastPage(hfile) hl_goTo(hfile->nb_scrlines,-1,hfile)
#define hl_goFirstPage(hfile) hl_goTo(0,-1,hfile)
#define hl_goLeft(hfile)
#define hl_goRight(hfile)

#define hl_goPageDown(hfile,common_band) {hfile->buffer.pos.y += hfile->screen.size.height - common_band;hl_updateBuffers(hfile);hl_refreshScreen(hfile);}
#define hl_goPageUp(hfile,common_band)  {hfile->buffer.pos.y += common_band - hfile->screen.size.height;hl_updateBuffers(hfile);hl_refreshScreen(hfile);}
#define hl_goPageLeft(hfile,common_band)
#define hl_goPageRight(hfile,common_band)

#define hl_canGoDown(hfile) (hfile->line_bot<hfile->nb_scrlines-1 || hfile->buffer.pos.y+hfile->screen.size.height<hfile->drawn_height)
#define hl_canGoUp(hfile) (hfile->line_top!=0 || hfile->buffer.pos.y>0)
#define hl_canGoLeft(hfile) (hfile->buffer.pos.x>0)
#define hl_canGoRight(hfile) (hfile->buffer.pos.x+hfile->screen.size.width<hfile->buffer.size.width)

/**************** Font ********************/
short hl_drawChar(h_Font * font, short x, short y, unsigned char caract, BOOL italic, BOOL bold, h_ScreenMem screen);

short hl_drawStr(h_Font * font, short x, short y, const unsigned char * str, BOOL italic, BOOL bold, h_ScreenMem screen);

short hl_charWidth(h_Font * font, unsigned char caract, BOOL italic, BOOL bold);

#define hl_italicWidth(font) ((font->upperline+font->underline)/3)

short hl_strWidth(h_Font * font, const unsigned char * str, BOOL italic, BOOL bold);

void hl_loadTiosFont(h_Font * font1,h_Font * font2,h_Font * font3);

short hl_loadAllFont(h_Font * * fonttab);

void hl_unloadAllFont(h_Font * fonttab, short nb_font);

h_Font * hl_findFont(const unsigned char * filename_font, h_Font * fonttab, short nb_font);

/**************** Menu ********************/
void hl_drawMenu(h_Menu * hmenu, h_ScreenMem screen);

void hl_goMenuBottom(h_Menu * hmenu);

void hl_goMenuPageDown(h_Menu * hmenu);

void hl_goMenuDown(h_Menu * hmenu);

void hl_goMenuTop(h_Menu * hmenu);

void hl_goMenuPageUp(h_Menu * hmenu);

void hl_goMenuUp(h_Menu * hmenu);

void hl_goSubMenu(h_Menu * hmenu);

void hl_goUpperMenu(h_Menu * hmenu);

#define hl_setMenuMove(hmenu)   (hmenu->move=1)
#define hl_resetMenuMove(hmenu)   (hmenu->move=0)
#define hl_changeMenuMove(hmenu)   (hmenu->move=1-hmenu->move)

/**************** Utils ********************/

short hl_addHANDLE(HANDLE * h, unsigned short * nb, unsigned short * size, unsigned short size_item);

void hl_drawLineVert(short x, short y1, short y2, short mode, h_ScreenMem screen);

void hl_fillFrame(short x1, short y1, short x2, short y2, short mode, h_ScreenMem screen);

#define hl_drawLineHoriz(x1,x2,y,mode,screen) hl_fillFrame(x1,y,x2,y,mode,screen)
#define hl_drawPixel(x,y,mode,screen) hl_drawLineVert(x,y,y,mode,screen);

#define hl_clrScreen(screen)   memset(screen.mem.ptr,0,screen.size.height*screen.mem.byte_width)

/**************** Bookmark ********************/

void hl_drawBookmark(h_File * hfile, short no_bkmk, short x, short y, short lcdwidth, h_ScreenMem screen, h_Font * font);

void hl_goTo(short num_line, short bookmark, h_File * hfile);

/**************** Links ********************/

short hl_firstLinkOfLine(short no_line, h_File * hfile);

short hl_getPrevLinkOfPage(short no_link, h_File * hfile);

short hl_getNextLinkOfPage(short no_link, h_File * hfile);

#define hl_getFirstLinkOfPage(hfile) hl_getNextLinkOfPage(hl_firstLinkOfLine(hfile->line_top,hfile)-1,hfile)
#define hl_getLastLinkOfPage(hfile) hl_firstLinkOfLine(hfile->line_bot+1,hfile)

BOOL hl_getTargetLink(unsigned char * name_var, short no_link, h_File * hfile);

h_LinkPos hl_getLinkPosition(short no_link, h_File * hfile);

short hl_getLink(short x, short y, h_File * hfile);


#define h_getAuthor(hfile)   (hfile->hh.pos_author==NOK ? NULL : hfile->text+hfile->hh.pos_author)
#define h_getComment(hfile)  (hfile->hh.pos_comment==NOK ? NULL : hfile->text+hfile->hh.pos_comment)
#define h_getDate(hfile)     (hfile->hh.pos_date==NOK ? NULL : hfile->text+hfile->hh.pos_date)
#define h_getTitle(hfile)    (hfile->hh.pos_title==NOK ? NULL : hfile->text+hfile->hh.pos_title)

#endif


